@@ -1,18 +1,22 @@
 # Copyright 2017 Palantir Technologies, Inc.
 import logging
 import socketserver
+import threading
+
+from jsonrpc.dispatchers import MethodDispatcher
+from jsonrpc.endpoint import Endpoint
+from jsonrpc.streams import JsonRpcStreamReader, JsonRpcStreamWriter
 
 from . import lsp, _utils, uris
 from .config import config
-from .jsonrpc.dispatchers import MethodDispatcher
-from .jsonrpc.endpoint import Endpoint
-from .jsonrpc.streams import JsonRpcStreamReader, JsonRpcStreamWriter
 from .workspace import Workspace
 
 log = logging.getLogger(__name__)
 
 
 LINT_DEBOUNCE_S = 0.5  # 500 ms
+PARENT_PROCESS_WATCH_INTERVAL = 10  # 10 s
+MAX_WORKERS = 64
 
 
 class _StreamHandlerWrapper(socketserver.StreamRequestHandler, object):
@@ -49,11 +53,11 @@ def start_tcp_lang_server(bind_addr, port, handler_class):
         server.server_close()
 
 
-def start_io_lang_server(rfile, wfile, handler_class):
+def start_io_lang_server(rfile, wfile, check_parent_process, handler_class):
     if not issubclass(handler_class, PythonLanguageServer):
         raise ValueError('Handler class must be an instance of PythonLanguageServer')
     log.info('Starting %s IO language server', handler_class.__name__)
-    server = handler_class(rfile, wfile)
+    server = handler_class(rfile, wfile, check_parent_process)
     server.start()
 
 
@@ -64,13 +68,14 @@ class PythonLanguageServer(MethodDispatcher):
 
     # pylint: disable=too-many-public-methods,redefined-builtin
 
-    def __init__(self, rx, tx):
+    def __init__(self, rx, tx, check_parent_process=False):
         self.workspace = None
         self.config = None
 
         self._jsonrpc_stream_reader = JsonRpcStreamReader(rx)
         self._jsonrpc_stream_writer = JsonRpcStreamWriter(tx)
-        self._endpoint = Endpoint(self, self._jsonrpc_stream_writer.write)
+        self._check_parent_process = check_parent_process
+        self._endpoint = Endpoint(self, self._jsonrpc_stream_writer.write, max_workers=MAX_WORKERS)
         self._dispatchers = []
         self._shutdown = False
 
@@ -148,10 +153,23 @@ class PythonLanguageServer(MethodDispatcher):
             rootUri = uris.from_fs_path(rootPath) if rootPath is not None else ''
 
         self.workspace = Workspace(rootUri, self._endpoint)
-        self.config = config.Config(rootUri, initializationOptions or {})
+        self.config = config.Config(rootUri, initializationOptions or {}, processId)
         self._dispatchers = self._hook('pyls_dispatchers')
         self._hook('pyls_initialize')
 
+        if self._check_parent_process and processId is not None:
+            def watch_parent_process(pid):
+                # exist when the given pid is not alive
+                if not _utils.is_process_alive(pid):
+                    log.info("parent process %s is not alive", pid)
+                    self.m_exit()
+                log.debug("parent process %s is still alive", pid)
+                threading.Timer(PARENT_PROCESS_WATCH_INTERVAL, watch_parent_process, args=[pid]).start()
+
+            watching_thread = threading.Thread(target=watch_parent_process, args=(processId,))
+            watching_thread.daemon = True
+            watching_thread.start()
+
         # Get our capabilities
         return {'capabilities': self.capabilities()}
 
